/**
  * INBOX25 Intelligence Panel
  *
  * @class View.Views.Base.Inbox25View
  * @alias SUGAR.App.view.views.Inbox25View
  * @extends View.DashletToolbarView
  */
({

  // use DashletToolbarView so we know the current loading/static dashlet icons
  extendsFrom: 'DashletToolbarView',

  _defaultSettings: {
    iframe_type: 'timeline'
  },

  inbox25_url: '',
  inbox25_url_ts: '',
  inbox25_url_record: '',

  plugins: ['Dashlet'],

  initialize: function(options) {
    this._super('initialize', [options]);
  },

  initDashlet: function() {
    this._super('initDashlet');
    this.settings.set('iframe_type', this.getType());
  },

  events: function() {
    var prototype = Object.getPrototypeOf(this);
    var parentEvents = _.result(prototype, 'events');

    return _.extend({}, parentEvents, {
      'click [data-action=type-switcher]': 'typeSwitcher'
    });
  },

  // toggle the loading icon on the dashboard's header
  // @see parent class DashletToolbarView method refreshClicked for inspiration
  toggleLoadingIcon: function(loading) {
    if (typeof loading === "undefined" || loading === null) {
      loading = true;
    }

    var $el = $(this.layout.$el[0]).find('[data-action=loading]');
    if(loading) {
      $el.removeClass(this.cssIconDefault).addClass(this.cssIconRefresh);
    }
    else {
      $el.removeClass(this.cssIconRefresh).addClass(this.cssIconDefault);
    }
  },

  // When User clicks the Vision or Timeline button, update the iframe_type
  // param ('vision' or 'timeline') and trigger a re-set of the INBOX25 URLs
  typeSwitcher: function(event) {
    var iframe_type = this.$(event.currentTarget).val();
    if (iframe_type === this.getType()) {
      return;
    }

    // show the loading icon
    this.toggleLoadingIcon(true);

    this.settings.set('iframe_type', iframe_type);
    this.setupURLs();
  },

  // get the current type value or return the default type
  getType: function() {
    return this.settings.get('iframe_type') || this._defaultSettings.iframe_type;
  },

  setupURLs: function() {
    // reset timestamp so we  know it is out of date
    this.inbox25_url_ts = '';

    // the INBOX25 URL
    this.inbox25_url = SUGAR.App.user.attributes.inbox25_url;

    if(this.inbox25_url != 'https://www.inbox25.com/timelinefail.php') {
      // we will pass the Sugar record reference to INBOX25 as a GET param, but
      // the param. changes contextually.
      // * to load the 'vision' view use sugar_id_vision
      // * to load the 'timeline' view for an Opp use sugar_id_opp
      // * to load the 'timeline' view for an person record use sugar_id
      if(this.getType() == 'vision') {
        this.inbox25_url_record = '&sugar_id_vision=';
      }
      else if(this.module == 'Opportunities') {
        this.inbox25_url_record = '&sugar_id_opp=';
      }
      else {
        this.inbox25_url_record = '&sugar_id=';
      }

      // whatever timeline/vision view we build, pass the record GUID
      this.inbox25_url_record += this.model.get('id');
      // then pass the module reference
      this.inbox25_url_record += '&sugar_module=';
      this.inbox25_url_record += this.module;
      // then pass the User ID
      this.inbox25_url_record += '&suid=' + SUGAR.App.user.id;
    }

    // The INBOX25 encoded timestamp should change every reload to avoid trying
    // to use an old one. We can get a new one by calling the customized API
    // endpoint '/me'.
    // (note that it would be fun to just do an SUGAR.App.user.load() but we
    // can't pass context into the callback function to get back to the dashlet
    // scope)
    var self = this;
    app.api.me('read', null, null, {
      success: function(data) {
        if (data && data.current_user && data.current_user.inbox25_url_ts) {
          self.inbox25_url_ts = data.current_user.inbox25_url_ts;
          self.loadData();
        }
      },
      error: function(err) {
        app.error.handleHttpError(err);
      }
    });
  },

  // load the dashlet, which includes checking the INBOX25 timestamp and setting
  // up the URLs if they're empty
  loadData: function(options) {
    if (this.disposed || this.meta.config) {
      return;
    }

    // if we don't have the iframe URL or timestamp yet (and we do have an
    // INBOX25 URL), queue that we need to get them before we try to do
    // anything further
    if(this.inbox25_url == '' || this.inbox25_url_ts == '') {
      this.setupURLs();
      return;
    }

    // turn off the loading icon
    this.toggleLoadingIcon(false);

    // in the new Sugar Fall 2018 UI, the active class isn't automatically
    // applied to the button, so we have to ensure that happens correctly here
    // even if the older version of Sugar handled it for us.
    $(this.layout.$el[0]).find('[data-action=type-switcher]').removeClass('active');
    $(this.layout.$el[0]).find('[data-action=type-switcher][value=' + this.getType() + ']').addClass('active');

    // update the iframe src, which should trigger a reload of the iframe
    var src = this.inbox25_url + this.inbox25_url_record;
    // only worry about the timestamp if we don't have a timelinefail.php src
    if(this.inbox25_url != 'https://www.inbox25.com/timelinefail.php'){
      src += this.inbox25_url_ts;
    }
    this.$('#current_iframe').attr('src', src);
  }

})
